==> open-xchange-guard_2.10.4.orig.tar.bz2: open-xchange-guard-2.10.4/com.openexchange.guard.mime.encryptor/src/com/openexchange/guard/mime/encryptor/pgp/pgpmime/GuardPGPMimeEncryptor.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mime.encryptor.pgp.pgpmime;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.mail.BodyPart;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMessage.RecipientType;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPSecretKey;
import com.openexchange.exception.OXException;
import com.openexchange.guard.common.util.CipherUtil;
import com.openexchange.guard.common.util.GuardMimeHeaders;
import com.openexchange.guard.exceptions.GuardCoreExceptionCodes;
import com.openexchange.guard.keymanagement.commons.GuardKeys;
import com.openexchange.guard.keymanagement.commons.RecipKey;
import com.openexchange.guard.keymanagement.commons.util.RecipKeyListUtils;
import com.openexchange.guard.keymanagement.services.GuardKeyService;
import com.openexchange.guard.mime.encryptor.Encryptor;
import com.openexchange.guard.mime.encryptor.exceptions.MimeEncryptorExceptionCodes;
import com.openexchange.guard.mime.encryptor.osgi.Services;
import com.openexchange.guard.mime.helpfiles.HelpFileService;
import com.openexchange.guard.mime.services.GuardParsedMimeMessage;
import com.openexchange.guard.pgpcore.services.PGPCryptoService;
import com.openexchange.pgp.keys.tools.PGPKeysUtil;
import com.openexchange.pgp.mail.PGPMimeService;


/**
 * Perform Mime encryption.
 * {@link GuardPGPMimeEncryptor}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.8.0
 */
public class GuardPGPMimeEncryptor extends Encryptor {

    private final HelpFileService helpFileService;

    public GuardPGPMimeEncryptor (PGPCryptoService pgpCryptoService,
                                  HelpFileService helpFileService,
                                  GuardParsedMimeMessage msg,
                                  ArrayList<RecipKey> recips,
                                  int userid,
                                  int cid,
                                  String password) {
        super (pgpCryptoService, msg, recips, userid, cid, password);
        this.helpFileService = Objects.requireNonNull(helpFileService, "helpFileService must not be null");
    }

    /**
     * Internal method to fetch the sender's PGP key
     *
     * @return The sender's key
     * @throws OXException
     */
    private GuardKeys getKey() throws OXException {
        GuardKeyService guardKeyService = Services.getService(GuardKeyService.class);
        return msg.senderIsGuest() ?
                guardKeyService.getKeys(msg.getSenderEmail()) :
                guardKeyService.getKeys(userid, cid);
    }

    @Override
    public MimeMessage doEncrypt () throws OXException {
        try {
            PGPEncryptedAttachmentManager pgpAttach = new PGPEncryptedAttachmentManager(msg, password);
            if (pgpAttach.hasEncryptedAttachment()) {
                msg.setMessage(pgpAttach.getDecoded());
            }

            GuardKeys key = getKey();
            if(key == null) {
                throw GuardCoreExceptionCodes.KEY_NOT_FOUND_FOR_IDS_ERROR.create(userid, cid);
            }
            RecipKey senderKey = new RecipKey(key);
            if(senderKey.getEncryptionKey() == null) {
                //Cannot find sender's key or use it was revoked or is expired
                throw GuardCoreExceptionCodes.KEY_NOT_FOUND_FOR_IDS_ERROR.create(userid, cid);
            }
            recips.add(senderKey);  // Add the sender here so it can be read in sent folder

            //Get additional help files to attach to the message
            final List<BodyPart> helpFiles  = helpFileService.attachHelpFile(userid, cid) ?
                helpFileService.getHelpFiles(userid, cid, RecipKeyListUtils.getLanguagesFor(recips)) :
                null;


            PGPMimeService pgpMimeService = Services.getService(PGPMimeService.class);
            MimeMessage ret = null;
            if (msg.getPIN() != null) {  // If pin added, add header within encrypted email for sender to check
                msg.getMessage().setHeader(GuardMimeHeaders.X_GUARD_PIN, msg.getPIN());
            }
            msg.getMessage().setRecipients(RecipientType.BCC, (String) null);  // remove BCC from Mime before encrypting
            if (msg.isSign() && !msg.isDraft()) {  // not signing Drafts
                char[] hashedPassword = CipherUtil.getSHA(password, key.getSalt()).toCharArray();
                ret = pgpMimeService.encryptSigned(msg.getMessage(), getSignerKey(key), hashedPassword, toPGPPublicKeys(recips), helpFiles);
            } else {
                ret = pgpMimeService.encrypt(msg.getMessage(), toPGPPublicKeys(recips), helpFiles);
            }

            return ret;
        } catch (Exception e) {
            throw GuardCoreExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
        }

    }

    /**
     * Extracts the PGP signing key from the given Guard key
     *
     * @param guardKey The key to get the singing key for
     * @return The signing key for the given Guard key
     * @throws OXException If no signing key was found
     */
    PGPSecretKey getSignerKey(GuardKeys guardKey) throws OXException {
        PGPSecretKey signingKey = PGPKeysUtil.getSigningKey(guardKey.getPGPSecretKeyRing());
        if (signingKey != null) {
            return signingKey;
        } else {
            //TODO: Throw Exception?
            return null;
        }
    }

    private List<PGPPublicKey> toPGPPublicKeys(List<RecipKey> recipients){
        //Getting recipient keys suitable for encryption
        List<PGPPublicKey> pgpRecipientKeysList = new ArrayList<PGPPublicKey>();
        for (RecipKey recipientKey : recipients) {
            PGPPublicKey encryptionKey = recipientKey.getEncryptionKey();
            if(encryptionKey != null) {
                pgpRecipientKeysList.add(encryptionKey);
            }
        }
        return pgpRecipientKeysList;
    }


    @Override
    public String doEncryptToString() throws OXException {
        ByteArrayOutputStream wrapped_out = new ByteArrayOutputStream();
        try {
            doEncrypt().writeTo(wrapped_out);
            byte[] wrapped_bytes = wrapped_out.toByteArray();
            return (new String (wrapped_bytes, "UTF-8"));
        } catch (IOException | MessagingException e) {
            throw MimeEncryptorExceptionCodes.SEND_EMAIL_ERROR.create(e.getMessage());
        }
    }
}

==> open-xchange-guard_2.10.5.orig.tar.bz2: open-xchange-guard-2.10.5/com.openexchange.guard.mime.encryptor/src/com/openexchange/guard/mime/encryptor/pgp/pgpmime/GuardPGPMimeEncryptor.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mime.encryptor.pgp.pgpmime;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.mail.BodyPart;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMessage.RecipientType;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPSecretKey;
import com.openexchange.exception.OXException;
import com.openexchange.guard.common.util.CipherUtil;
import com.openexchange.guard.common.util.GuardMimeHeaders;
import com.openexchange.guard.exceptions.GuardCoreExceptionCodes;
import com.openexchange.guard.keymanagement.commons.GuardKeys;
import com.openexchange.guard.keymanagement.commons.RecipKey;
import com.openexchange.guard.keymanagement.commons.util.RecipKeyListUtils;
import com.openexchange.guard.keymanagement.services.GuardKeyService;
import com.openexchange.guard.mime.encryptor.Encryptor;
import com.openexchange.guard.mime.encryptor.exceptions.MimeEncryptorExceptionCodes;
import com.openexchange.guard.mime.encryptor.osgi.Services;
import com.openexchange.guard.mime.helpfiles.HelpFileService;
import com.openexchange.guard.mime.services.GuardParsedMimeMessage;
import com.openexchange.guard.pgpcore.services.PGPCryptoService;
import com.openexchange.pgp.keys.tools.PGPKeysUtil;
import com.openexchange.pgp.mail.PGPMimeService;


/**
 * Perform Mime encryption.
 * {@link GuardPGPMimeEncryptor}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.8.0
 */
public class GuardPGPMimeEncryptor extends Encryptor {

    private final HelpFileService helpFileService;
    private int oxUserId, oxCid;

    public GuardPGPMimeEncryptor (PGPCryptoService pgpCryptoService,
                                  HelpFileService helpFileService,
                                  GuardParsedMimeMessage msg,
                                  ArrayList<RecipKey> recips,
                                  int userid,
                                  int cid,
        int oxUserId,
        int oxCid,
                                  String password) {
        super (pgpCryptoService, msg, recips, userid, cid, password);
        this.oxUserId = oxUserId;
        this.oxCid = oxCid;
        this.helpFileService = Objects.requireNonNull(helpFileService, "helpFileService must not be null");
    }

    /**
     * Internal method to fetch the sender's PGP key
     *
     * @return The sender's key
     * @throws OXException
     */
    private GuardKeys getKey() throws OXException {
        GuardKeyService guardKeyService = Services.getService(GuardKeyService.class);
        return msg.senderIsGuest() ?
                guardKeyService.getKeys(msg.getSenderEmail()) :
                guardKeyService.getKeys(userid, cid);
    }

    @Override
    public MimeMessage doEncrypt () throws OXException {
        try {
            PGPEncryptedAttachmentManager pgpAttach = new PGPEncryptedAttachmentManager(msg, password);
            if (pgpAttach.hasEncryptedAttachment()) {
                msg.setMessage(pgpAttach.getDecoded());
            }

            GuardKeys key = getKey();
            if(key == null) {
                throw GuardCoreExceptionCodes.KEY_NOT_FOUND_FOR_IDS_ERROR.create(userid, cid);
            }
            RecipKey senderKey = new RecipKey(key);
            if(senderKey.getEncryptionKey() == null) {
                //Cannot find sender's key or use it was revoked or is expired
                throw GuardCoreExceptionCodes.KEY_NOT_FOUND_FOR_IDS_ERROR.create(userid, cid);
            }
            recips.add(senderKey);  // Add the sender here so it can be read in sent folder

            //Get additional help files to attach to the message
            final List<BodyPart> helpFiles = helpFileService.attachHelpFile(oxUserId, oxCid) ? helpFileService.getHelpFiles(oxUserId, oxCid, RecipKeyListUtils.getLanguagesFor(recips)) :
                null;


            PGPMimeService pgpMimeService = Services.getService(PGPMimeService.class);
            MimeMessage ret = null;
            if (msg.getPIN() != null) {  // If pin added, add header within encrypted email for sender to check
                msg.getMessage().setHeader(GuardMimeHeaders.X_GUARD_PIN, msg.getPIN());
            }
            if (!msg.isDraft()) {
                msg.getMessage().setRecipients(RecipientType.BCC, (String) null);  // remove BCC from Mime before encrypting
            }

            if (msg.isSign() && !msg.isDraft()) {  // not signing Drafts
                char[] hashedPassword = CipherUtil.getSHA(password, key.getSalt()).toCharArray();
                ret = pgpMimeService.encryptSigned(msg.getMessage(), getSignerKey(key), hashedPassword, toPGPPublicKeys(recips), helpFiles);
            } else {
                ret = pgpMimeService.encrypt(msg.getMessage(), toPGPPublicKeys(recips), helpFiles);
            }

            return ret;
        } catch (Exception e) {
            throw GuardCoreExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
        }

    }

    /**
     * Extracts the PGP signing key from the given Guard key
     *
     * @param guardKey The key to get the singing key for
     * @return The signing key for the given Guard key
     * @throws OXException If no signing key was found
     */
    PGPSecretKey getSignerKey(GuardKeys guardKey) throws OXException {
        PGPSecretKey signingKey = PGPKeysUtil.getSigningKey(guardKey.getPGPSecretKeyRing());
        if (signingKey != null) {
            return signingKey;
        } else {
            //TODO: Throw Exception?
            return null;
        }
    }

    private List<PGPPublicKey> toPGPPublicKeys(List<RecipKey> recipients){
        //Getting recipient keys suitable for encryption
        List<PGPPublicKey> pgpRecipientKeysList = new ArrayList<PGPPublicKey>();
        for (RecipKey recipientKey : recipients) {
            PGPPublicKey encryptionKey = recipientKey.getEncryptionKey();
            if(encryptionKey != null) {
                pgpRecipientKeysList.add(encryptionKey);
            }
        }
        return pgpRecipientKeysList;
    }


    @Override
    public String doEncryptToString() throws OXException {
        ByteArrayOutputStream wrapped_out = new ByteArrayOutputStream();
        try {
            doEncrypt().writeTo(wrapped_out);
            byte[] wrapped_bytes = wrapped_out.toByteArray();
            return (new String (wrapped_bytes, "UTF-8"));
        } catch (IOException | MessagingException e) {
            throw MimeEncryptorExceptionCodes.SEND_EMAIL_ERROR.create(e.getMessage());
        }
    }
}
